package UroIrmPackage;
	
import ij.*;
import ij.io.*;
import ij.gui.*;
import ij.text.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

/** 	
	display the result like "ij.plugin.filter.Analyzer" 
*/
public class UroIrmResultDisplay extends Frame implements ActionListener, FocusListener,UroIrmMessage  {
	private TextPanel textPanel;
	private UroIrmSaveResult sr;	
	private PrintWriter pw;
	private Toolkit tk;
	ImagePlus imp;
	private String title;

	private static String name_class = "Result " ;
	private static boolean unsaved = true;
	private static boolean isquit = true;
	

	public UroIrmResultDisplay() {
		super(name_class);
		ImageJ ij = IJ.getInstance();
		if (ij!=null) {
			Image img = ij.getIconImage();
			if (img!=null) setIconImage(img);
		}
		imp= getImage();
		if ( imp == null)
			return;
		int nSlices = imp.getStackSize();
		if ( nSlices < 1 ) { 
			return ;
		}
		if ( title == null)
			title = name_class;
		init();
	}

	private void init(){
		tk = Toolkit.getDefaultToolkit();
		Dimension dimScreen = tk.getScreenSize();
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		textPanel = new TextPanel();
		textPanel.append("");
		add("Center", textPanel);
		sr = new UroIrmSaveResult(imp,textPanel);
		ImageJ ij = IJ.getInstance();
		if (ij!=null) {
			Image img = ij.getIconImage();
			if (img!=null) setIconImage(img);
		}
		addFocusListener(this);
		addMenuBar();
		WindowManager.addWindow(this);
		setSize(500,dimScreen.height/2);
		this.setLocation(dimScreen.width-500,0);
		unsaved = true;
		isquit = false;
	}

/** tells the class that the data were not saved */
	public void setUnsaved(){
		unsaved = true;
	}	
	
/** return true if the data were saved */
	public boolean getUnsaved(){
		return unsaved;
	}

	public Toolkit getToolkit(){
		return tk;
	}	

	public Frame getFrame(){
		return this;
	}
	
	public void addEvaluation(float[] point1,float[] point2,float res){
		sr.addEvaluation(point1,point2,res);
	}

	public void setTitle(String arg){
		title = arg;
		super.setTitle("Results of " + arg);
		sr.setTitle(arg);
	}

	private ImagePlus getImage() {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) {
			error(msg_err_no_image);
			return null;
		} else
			return imp;
	}

/** update the data and make the frame visible */		
	public void upDateAndVisible(){
		if ( !isVisible())
			setVisible(true);
		upDate();
	}

/** update only the data */
	public void upDate(){
		unsaved=true;
		if (!isVisible())
			return;
		clearWorkSheet();
		if ( textPanel == null){
			textPanel = new TextPanel();
			sr = new UroIrmSaveResult(imp,textPanel);
		}
		textPanel.append("");
		displayResult();
		add("Center", textPanel);
	}

	private void addMenuBar() {
		MenuBar mb = new MenuBar();
		Menu m = new Menu("File");
		m.add(new MenuItem("Save As..."/*, new MenuShortcut(KeyEvent.VK_S)*/));
		m.addActionListener(this);
		mb.add(m);
		m = new Menu("Edit");
		m.add(new MenuItem("Summarize"));
		m.add(new MenuItem("Set Measurements..."));
		
		m.addActionListener(this);
		mb.add(m);
		setMenuBar(mb);
	}

/* return the text put on the frame */
	public String getText( ){
		int nb = textPanel.getLineCount();
		String outPut = "";
		if ( nb < 0 ) 
			return null;
		for ( int line = 0 ; line < nb ;line ++){
			outPut += textPanel.getLine(line)+ " \n " ;
		}
		return outPut;	
	}

/** return the headings of the frame; */
	public String getHeadings(){
		return textPanel.getColumnHeadings();
	}

/** add a text to the current text */
	public void append(String text) {
		textPanel.append(text);
	}
	
/** Set the font that will be used to display the text. */
	public void setFont(Font font) {
		textPanel.setFont(font);
	}
  
/** Returns a reference to this TextWindow's TextPanel. */
	public TextPanel getTextPanel() {
		return textPanel;
	}
	
	public void actionPerformed(ActionEvent evt) {
		String cmd = evt.getActionCommand();
		doCommand(cmd);
	}

	void doCommand(String cmd) {
 		if (cmd==null)
 			return;
		if (cmd.equals("Save As..."))
			   savePath();
		else if (cmd.equals("Summarize")){
			sr.summarize();
		}
		else if (cmd.equals("Set Measurements...")) {
			setMeasurements();
		}
	}
	
	private void setMeasurements(){
		sr.doSetDialog() ; 
		sr.update();
		int nSlices = imp.getStackSize();
		int currentSlice = imp.getCurrentSlice();
		for ( int slice = 1 ; slice < nSlices+1 ; slice++ ) {
			imp.setSlice(slice);
			sr.measure();
		}
		imp.setSlice(currentSlice);
		sr.displayResults();
	}

	void savePath(){
		if ( !saveAs(""))
 			unsaved = true;
 		else
 			unsaved = false;
	}

	private void save(PrintWriter pw) {
		String headline = getHeadings();
		String outPut = getText();
		String aux = outPut;
		int index =-1;
		if (outPut!=null && !outPut.equals("")){
				index = outPut.indexOf(UroIrmSaveResult.MARK_DATA);
				if ( index > 0 ) {
					pw.println(outPut.substring(0,index));
					outPut = outPut.substring(index+UroIrmSaveResult.MARK_DATA.length());
				}else if ( index >=0){
					outPut = outPut.substring(index+UroIrmSaveResult.MARK_DATA.length());
				}
		}
		if (headline!=null && !headline.equals(""))
			pw.println(headline);
		if (outPut!=null && !outPut.equals(""))
			pw.println(outPut);
	}
	
/** save the result in a file given by "path" if path == "" then a SaveDialog is launch*/
	public boolean saveAs(String path) {
		String file = "" ;
		if (path.equals("")) {
			String txt = (title.equals(""))? name_class:title; 
			SaveDialog sd = new SaveDialog("Save as Result",txt,".txt");
			file = sd.getFileName();
			if (file == null || file.equals("") ) return false;
			path = sd.getDirectory() + file;
		}
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return false;
		}
		save(pw);
		pw.close();
		IJ.showStatus(file + " saved ");
		unsaved=false;
		return true;
	}

/** display the data */
	public void displayResult(){
		int nSlices = imp.getStackSize();
		int currentSlice = imp.getCurrentSlice();
		for ( int slice = 1 ; slice < nSlices+1 ; slice++ ) {
			imp.setSlice(slice);
			sr.measure();
		}
		imp.setSlice(currentSlice);		
		sr.displayResults();
	}

/** clear the worksheet */
	public void clearWorkSheet() {
		unsaved = true;
		if ( sr ==null) {
			return;
		}
		sr.setSaved();
		sr.resetCounter();
	}

/** same as Quit but it indicate that the frame won't be called anymore. */
	public void end(){
	 	if ( ! isVisible())
			setVisible(true);
		isquit = true;
		Quit();
	}

/** quit the frame */
	public void Quit(){
		 if ( unsaved ) {
			clearWorkSheet();
			upDate();
		 	if (!saveAs(""))
				return;
		}
		clearWorkSheet();
		setVisible(false);
		dispose();
		WindowManager.removeWindow(this);
		textPanel = null;
		sr=null;
		PrefSaveResult.save();
		PrefSaveResult.flush();
	}
		
/** if end() was called then the action will call quit() otherwise 
	it clear the frame */
	public void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if (e.getID()==WindowEvent.WINDOW_CLOSING) {
			if ( isquit ){ 
				if( IJ.showMessageWithCancel("QUIT",msg_err_no_data_save +"\n"+msg_question_save_result)){
					Quit();
					return;
				}
			}
			dispose();
			PrefSaveResult.save();
			PrefSaveResult.flush();
		
		}
	}

	public void focusGained(FocusEvent e) {
		WindowManager.setWindow(this);
	}

	public void focusLost(FocusEvent e) {}

/** remove all */
	public void flush(){
		sr.flush();
		sr =null;textPanel=null;pw=null;tk=null;imp=null;
	}

// fonction pour les erreurs specifiques a la classe
	private boolean error(String msg) {
		new MessageDialog((new Frame()),name_class , msg);
		return false;
	}

}


