import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;
import ij.io.*;
import ij.plugin.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*; 
import java.awt.image.*; 

/*
	* The class name and file name must be the same.
	* Plugin filters such as this one automatically get Undo support.
	
*/

public class UroIrmRecal_ implements PlugInFilter, ActionListener  {

ImagePlus imp;			// Current image
Frame framePoint ;	// Frame to display waiting messages

	public int setup(String arg, ImagePlus imp) {

		this.imp=imp;
		return DOES_8G+DOES_16+SUPPORTS_MASKING;	// if SUPPORTS_STACK is set up, run() is called for each slice
								// in a stack 
	}

	public void run(ImageProcessor ip) {


/******************************************************
*		transform 16 bit images to 8 bit
*		in order to keep the adjustment
*		of brightness and contrast
******************************************************/

		imp.unlock();
		IJ.run("8-bit");


/*****************************************************
*
*		boite de dialogue
*
******************************************************/
		
		Frame frame = new Frame("Recalage");	
		
		frame.setLocation(380,160);
	   	frame.setSize(180,125);
	    
		Panel Line1 = new Panel();
            Button buttonHigh = new Button("H");
		buttonHigh.addActionListener(this);
 		Line1.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		Line1.add(buttonHigh);
		frame.add("North", Line1);


		Panel Line2 = new Panel();
		Line2.setLayout(new FlowLayout(FlowLayout.CENTER,3,5));
		Label label = new Label("<-------|------->",Label.CENTER);
        	Button buttonLeft = new Button("L");
            Button buttonRight = new Button("R");
		buttonLeft.addActionListener(this);
		buttonRight.addActionListener(this);
            Line2.add(buttonLeft);
		Line2.add(label);
		Line2.add(buttonRight);
       	frame.add("Center",Line2);  

		Panel Line3 = new Panel();
            Button buttonBottom = new Button("B");
		Button buttonFinish = new Button("Stop");
		buttonBottom.addActionListener(this);
		buttonFinish.addActionListener(this);
		Line3.add(new Label("               "));
            Line3.add(buttonBottom);
		Line3.add(new Label(" "));
		Line3.add(buttonFinish);
        	frame.add("South", Line3);


		framePoint = frame ;
		framePoint.setVisible(true); 


	}

/********************************************************************************
*			actionPerformed						
*   CALLED by clicking on button in the message recalage		
*										
*   Calls the translate function
********************************************************************************/


	public void actionPerformed(ActionEvent e) {

		String label = e.getActionCommand();
	
		if (label==null)
			return;
		
		if (label.equals("H")){
			goHigh();
		}
		else if (label.equals("L")){
			goLeft();
			}
		else if (label.equals("R")){
			goRight();
			}	
		else if (label.equals("B")){
			goBottom();
			}
		else if (label.equals("Stop")){
			framePoint.setVisible(false);
			framePoint.dispose();
		}
	}

/*********************************************************
*			goHigh
*
*	translate the image to the up
*
**********************************************************/

	public void goHigh (){
		
		int currentSlice = imp.getCurrentSlice();
		ImageProcessor ip = imp.getStack().getProcessor(currentSlice);

		int width = imp.getWidth();
		int height= imp.getHeight();
		int[] data = new int[width];

		for(int i=0;i<height-1;i++)
		{
			ip.getRow(0,i+1,data,width);
			ip.putRow(0,i,data,width);
		}
		for(int i=0;i<width;i++) data[i]=0;
		ip.putRow(0,height-1,data,width);
		imp.setSlice(currentSlice);

	}

/*********************************************************
*			goBottom
*
*	translate the image to the down
*
**********************************************************/

	public void goBottom (){
		
		int currentSlice = imp.getCurrentSlice();
		ImageProcessor ip = imp.getStack().getProcessor(currentSlice);

		int width = imp.getWidth();
		int height= imp.getHeight();
		int[] data = new int[width];

		for(int i=height-1;i>0;i--)
		{
			ip.getRow(0,i-1,data,width);
			ip.putRow(0,i,data,width);
		}
		for(int i=0;i<width;i++) data[i]=0;
		ip.putRow(0,0,data,width);
		imp.setSlice(currentSlice);

	}

/*********************************************************
*			goLeft
*
*	translate the image to the left
*
**********************************************************/

	public void goLeft (){
		
		int currentSlice = imp.getCurrentSlice();
		ImageProcessor ip = imp.getStack().getProcessor(currentSlice);

		int width = imp.getWidth();
		int height= imp.getHeight();
		int[] data = new int[width];

		for(int i=0;i<width-1;i++)
		{
			ip.getColumn(i+1,0,data,height);
			ip.putColumn(i,0,data,height);
		}
		for(int i=0;i<height;i++) data[i]=0;
		ip.putColumn(width-1,0,data,height);
		imp.setSlice(currentSlice);

	}

/*********************************************************
*			goRight
*
*	translate the image to the right
*
**********************************************************/

	public void goRight (){
		
		int currentSlice = imp.getCurrentSlice();
		ImageProcessor ip = imp.getStack().getProcessor(currentSlice);

		int width = imp.getWidth();
		int height= imp.getHeight();
		int[] data = new int[width];

		for(int i=width-1;i>0;i--)
		{
			ip.getColumn(i-1,0,data,height);
			ip.putColumn(i,0,data,height);
		}
		for(int i=0;i<height;i++) data[i]=0;
		ip.putColumn(0,0,data,height);
		imp.setSlice(currentSlice);

	}



}

